# coding: utf-8

# flake8: noqa
"""
    OpenAPI spec version: 1.0.0
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

from __future__ import absolute_import

# import models into model package
from graphql_client.models.no_schema import NoSchema
